# LOCK-BEGIN[imports]: DON'T MODIFY
from podite import (
    AutoTagType,
    Enum,
    pod,
)

# LOCK-END


# LOCK-BEGIN[class(AccountTag)]: DON'T MODIFY
@pod
class AccountTag(Enum[AutoTagType]):
    UNINITIALIZED = None
    MARKET_PRODUCT_GROUP = None
    TRADER_RISK_GROUP = None
    TRADER_POSITION = None
    MARKET_PRODUCT_GROUP_WITH_COMBOS = None
    COMBO_GROUP = None
    COMBO = None
    RISK_PROFILE = None
    # LOCK-END

    @classmethod
    def _to_bytes_partial(cls, buffer, obj, **kwargs):
        # to modify packing, change this method
        return super()._to_bytes_partial(buffer, obj, **kwargs)

    @classmethod
    def _from_bytes_partial(cls, buffer, **kwargs):
        # to modify unpacking, change this method
        return super()._from_bytes_partial(buffer, **kwargs)

    @classmethod
    def to_bytes(cls, obj, **kwargs):
        return cls.pack(obj, converter="bytes", **kwargs)

    @classmethod
    def from_bytes(cls, raw, **kwargs):
        return cls.unpack(raw, converter="bytes", **kwargs)
